#pragma once

#include <cstddef>
#include <cstdint>
#include <type_traits>
#include <algorithm>

namespace mc::psx {
template <bool Const>
class BasicFrame {
	using data_type = typename std::conditional<Const, const uint8_t, uint8_t>::type;
public:
	static const constexpr unsigned int Size = 128;

	BasicFrame (data_type* data, std::size_t index);
	BasicFrame (const BasicFrame&) = default;
	BasicFrame (BasicFrame&&) = default;

	data_type* begin() { return m_data; }
	data_type* end() { return m_data + size(); }
	const data_type* cbegin() const { return m_data; }
	const data_type* cend() const { return m_data + size(); }
	const data_type* begin() const { return m_data; }
	const data_type* end() const { return m_data + size(); }
	data_type& operator[] (std::size_t index);
	const data_type& operator[] (std::size_t index) const;

	static constexpr std::size_t size() { return Size; }

	data_type* data() { return m_data; }
	const data_type* data() const { return m_data; }

	template <typename T>
	T extract (std::size_t offset) const;

private:
	void extract (std::size_t offset, std::size_t sz, uint8_t* out);

	std::size_t m_index;
	data_type* m_data;
};

template <typename T, bool Const>
inline T read_as (const BasicFrame<Const>& frame, std::size_t offset) {
	T retval;
	std::copy(
		frame.begin() + offset,
		frame.begin() + offset + sizeof(T),
		reinterpret_cast<uint8_t*>(&retval)
	);
	return retval;
}

using Frame = BasicFrame<false>;
using ConstFrame = BasicFrame<true>;
} //namespace mc::psx
