#include "memcard/memorycard.hpp"
#include "memcard/block.hpp"
#include "memcard/frame.hpp"
#include <iterator>
#include <cassert>

namespace mc::psx {
namespace {
	const constexpr std::size_t MemoryCardSize = 8192 * 16;

	template <bool Const>
	BasicBlock<Const> make_block_for_index (typename BasicBlock<Const>::data_type* data, std::size_t index) {
		const auto retptr = data + index * Block::size() * Frame::size();
		if (index > 0) {
			return {retptr, BasicFrame<Const>(data + index * Frame::size(), index), index};
		}
		else {
			return {data + index, index};
		}
	}
} //unnamed namespace

MemoryCard::MemoryCard() :
	m_data(MemoryCardSize)
{
}

MemoryCard::~MemoryCard() = default;

Block MemoryCard::operator[] (std::size_t index) {
	assert(index < 15);
	return block_at_index(index + 1);
}

ConstBlock MemoryCard::operator[] (std::size_t index) const {
	assert(index < 15);
	return block_at_index(index + 1);
}

ContentInfo MemoryCard::content_info() const {
	return {block_at_index(0).frame(0)};
}

std::size_t MemoryCard::size() const {
	return std::count_if(
		const_iterator(this, 0),
		const_iterator(this, 15),
		[](const auto& blk) {return blk.has_magic();}
	);
}

auto MemoryCard::begin() -> iterator {
	return iterator(this, 0);
}

auto MemoryCard::end() -> iterator {
	return iterator(this, size());
}

auto MemoryCard::cbegin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::begin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::cend() const -> const_iterator {
	return const_iterator(this, size());
}

auto MemoryCard::end() const -> const_iterator {
	return const_iterator(this, size());
}

Block MemoryCard::block_at_index(std::size_t index) {
	return make_block_for_index<false>(m_data.data(), index);
}

ConstBlock MemoryCard::block_at_index(std::size_t index) const {
	return make_block_for_index<true>(m_data.data(), index);
}

std::vector<std::size_t> block_group (const MemoryCard& mc, std::size_t index) {
	std::vector<std::size_t> retval;

	auto block = mc[index];
}
} //namespace mc::psx
